"""Verification of the correctness of the step-by-step execution of chunked
prefill with prefix caching. It does so by comparing, at every engine step 
(i.e. prefill or decode iteration), a bunch of attributes. 
This allows a finer testing of the padding and scheduling implementation.

Run `python -m pytest tests/e2e/test_spyre_pc_inference_steps.py`.
"""

import pytest
from scheduling_utils import check_scheduler_inference_steps
from spyre_util import ModelInfo


@pytest.mark.cpu
@pytest.mark.chunked_prefill
@pytest.mark.full_model
@pytest.mark.prefix_caching
# These values are all parameterized for test sorting
@pytest.mark.parametrize("max_num_seqs", [2])
@pytest.mark.parametrize("max_model_len", [256])
@pytest.mark.parametrize("max_num_batched_tokens", [128])
@pytest.mark.parametrize("available_blocks", [None])
def test_prefix_hit_within_batch(model: ModelInfo, backend: str,
                                 monkeypatch: pytest.MonkeyPatch,
                                 set_random_seed, max_num_seqs: int,
                                 max_model_len: int,
                                 max_num_batched_tokens: int,
                                 available_blocks: int):
    """ Scenario where two equal sequences are scheduled. 
    While prefilling the second sequence we have a prefix cache
    hit and can reuse the first chunk. Note that the fetched prefix blocks 
    are still part of the existing decode batch. Hence we have duplicated 
    blocks in the block table for this example.

    Configuration:
        * max_num_seqs: 2
        * number of prompts: 2
            * 0: len = 192,  max tokens = 2, step joining = 0
            * 1: len = 192, max tokens = 2, step joining = 0
    """
    monkeypatch.setenv("VLLM_SPYRE_CP_INTERLEAVE_STEPS", "0")

    seqs_max_tokens = [2, 2]
    prompts_lengths = [192, 192]
    steps_add_reqs = [0, 0]
    seeds = [0, 0]  # twice the same sequence

    checked_steps = [
        {
            "step": 0,
            "tkv": 0,
            "waiting": ["0", "1"],
            "running": [],
            "request_outputs": [],
            "n_reserved_blocks": 0,
            "n_used_blocks": 0
        },
        {   # prefill chunk 1 seq 0
            "step": 1,
            "tkv": 192,
            "waiting": ["1"],
            "running": ["0"],
            "request_outputs": [],
            "n_reserved_blocks": 4,
            "n_used_blocks": 3,
            "n_prefix_hits": 0,
        },
        {   # prefill chunk 2 seq 0
            "step": 2,
            "tkv": 192,
            "waiting": ["1"],
            "running": ["0"],
            "request_outputs": ["0"],
            "n_reserved_blocks": 4,
            "n_used_blocks": 3,
            "n_prefix_hits": 0,
        },
        {   # prefill chunk 1 seq 1
            # prefix hit!
            "step": 3,
            "tkv": 192,
            "waiting": [],
            "running": ["1", "0"],
            "request_outputs": [],
            "n_reserved_blocks": 8,
            "n_used_blocks": 6,
            "n_prefix_hits": 1,
            # each chunk has two blocks. Due to padding, the first chunk has
            # only one usable block
            "n_cached_blocks": 1
        },
        {   # prefill chunk 2 seq 1
            # cannot use prefix, as the last chunk has to always be recomputed
            "step": 4,
            "tkv": 192,
            "waiting": [],
            "running": ["1", "0"],
            "request_outputs": ["1"],
            "n_reserved_blocks": 8,
            "n_used_blocks": 6,
            "n_prefix_hits": 0,
            "n_cached_blocks": 1
        },

        {
            # Decode 1 of request 0.
            # Decode 1 of request 1.
            "step": 5,
            "tkv": 193,
            "waiting": [],
            "running": [],
            "request_outputs": ["1", "0"],
            "finished_requests": ["1", "0"],
            "n_reserved_blocks": 8,
            "n_used_blocks": 8,
            "n_cached_blocks": 1
        },
        {
            # Tkv should be cleared one step later
            "step": 6,
            "tkv": 0,
            "waiting": [],
            "running": [],
            "request_outputs": [],
            "n_reserved_blocks": 0,
            "n_used_blocks": 0
        },
    ]

    check_scheduler_inference_steps(
        model=model,
        backend=backend,
        monkeypatch=monkeypatch,
        seqs_max_tokens=seqs_max_tokens,
        prompts_lengths=prompts_lengths,
        steps_add_reqs=steps_add_reqs,
        checked_steps=checked_steps,
        max_num_seqs=max_num_seqs,
        max_model_len=max_model_len,
        available_blocks=available_blocks,
        use_cb=False,
        random_prompts=True,
        max_num_batched_tokens=max_num_batched_tokens,
        prefix_caching=True,
        seeds=seeds,
    )


@pytest.mark.cpu
@pytest.mark.chunked_prefill
@pytest.mark.full_model
@pytest.mark.prefix_caching
# These values are all parameterized for test sorting
@pytest.mark.parametrize("max_num_seqs", [2])
@pytest.mark.parametrize("max_model_len", [256])
@pytest.mark.parametrize("max_num_batched_tokens", [128])
@pytest.mark.parametrize("available_blocks", [None])
def test_prefix_hit_not_in_batch(model: ModelInfo, backend: str,
                                 monkeypatch: pytest.MonkeyPatch,
                                 set_random_seed, max_num_seqs: int,
                                 max_model_len: int,
                                 max_num_batched_tokens: int,
                                 available_blocks: int):
    """ Scenario where two equal sequences are scheduled. 
    While prefilling the second sequence we have a prefix cache
    hit and can reuse the first chunk. Note that the fetched prefix blocks 
    are not part of the existing decode batch as the sequence has already 
    left the batch at the time of prefilling the new sequence. Hence we have 
    no duplicated blocks in the block table for this example.

    Configuration:
        * max_num_seqs: 2
        * number of prompts: 2
            * 0: len = 192,  max tokens = 2, step joining = 0
            * 1: len = 192, max tokens = 2, step joining = 3
    """
    monkeypatch.setenv("VLLM_SPYRE_CP_INTERLEAVE_STEPS", "0")

    seqs_max_tokens = [2, 2]
    prompts_lengths = [192, 192]
    # sequence 1 joins only at step 3, when seq 0 has already finished
    steps_add_reqs = [0, 3]
    seeds = [0, 0]  # twice the same sequence

    checked_steps = [
        {
            "step": 0,
            "tkv": 0,
            "waiting": ["0"],
            "running": [],
            "request_outputs": [],
            "n_reserved_blocks": 0,
            "n_used_blocks": 0
        },
        {   # prefill chunk 1 seq 0
            "step": 1,
            "tkv": 192,
            "waiting": [],
            "running": ["0"],
            "request_outputs": [],
            "n_reserved_blocks": 4,
            "n_used_blocks": 3,
            "n_prefix_hits": 0,
        },
        {   # prefill chunk 2 seq 0
            "step": 2,
            "tkv": 192,
            "waiting": [],
            "running": ["0"],
            "request_outputs": ["0"],
            "n_reserved_blocks": 4,
            "n_used_blocks": 3,
            "n_prefix_hits": 0,
        },
        {
            # Decode 1 of request 0.
            # request 1 joined the waiting queue
            "step": 3,
            "tkv": 193,
            "waiting": ["1"],
            "running": [],
            "request_outputs": ["0"],
            "finished_requests": ["0"],
            "n_reserved_blocks": 4,
            "n_used_blocks": 4
        },
        {   # prefill chunk 1 seq 1
            # prefix hit!
            "step": 4,
            "tkv": 192,
            "waiting": [],
            "running": ["1"],
            "request_outputs": [],
            "n_reserved_blocks": 4,
            "n_used_blocks": 3,
            "n_prefix_hits": 1,
            "n_cached_blocks": 1
        },
        {   # prefill chunk 2 seq 1
            # cannot use prefix, as the last chunk has to always be recomputed
            "step": 5,
            "tkv": 192,
            "waiting": [],
            "running": ["1"],
            "request_outputs": ["1"],
            "n_reserved_blocks": 4,
            "n_used_blocks": 3,
            "n_prefix_hits": 0,
            "n_cached_blocks": 1
        },
        {
            # Decode 1 of request 0.
            "step": 6,
            "tkv": 193,
            "waiting": [],
            "running": [],
            "request_outputs": ["1"],
            "finished_requests": ["1"],
            "n_reserved_blocks": 4,
            "n_used_blocks": 4,
            "n_cached_blocks": 1
        },
        {
            # Tkv should be cleared one step later
            "step": 7,
            "tkv": 0,
            "waiting": [],
            "running": [],
            "request_outputs": [],
            "n_reserved_blocks": 0,
            "n_used_blocks": 0
        },
    ]

    check_scheduler_inference_steps(
        model=model,
        backend=backend,
        monkeypatch=monkeypatch,
        seqs_max_tokens=seqs_max_tokens,
        prompts_lengths=prompts_lengths,
        steps_add_reqs=steps_add_reqs,
        checked_steps=checked_steps,
        max_num_seqs=max_num_seqs,
        max_model_len=max_model_len,
        available_blocks=available_blocks,
        use_cb=False,
        random_prompts=True,
        max_num_batched_tokens=max_num_batched_tokens,
        prefix_caching=True,
        seeds=seeds,
    )


@pytest.mark.cpu
@pytest.mark.chunked_prefill
@pytest.mark.full_model
@pytest.mark.prefix_caching
# These values are all parameterized for test sorting
@pytest.mark.parametrize("max_num_seqs", [2])
@pytest.mark.parametrize("max_model_len", [256])
@pytest.mark.parametrize("max_num_batched_tokens", [128])
@pytest.mark.parametrize("available_blocks", [4])
def test_limit_blocks_no_prefix_hit(model: ModelInfo, backend: str,
                                    monkeypatch: pytest.MonkeyPatch,
                                    set_random_seed, max_num_seqs: int,
                                    max_model_len: int,
                                    max_num_batched_tokens: int,
                                    available_blocks: int):
    """ Scenario where three sequences are scheduled with the 1st and 3rd
    sequences being identical. While prefilling the third sequence we don't 
    have a prefix cache hit for the first chunk as the KV cache has already 
    been overwritten. This is because we limit the number of available blocks
    to 4. Note: When increasing the number of available blocks to 8, see
    test_limit_blocks_prefix_hit, the same test results in a prefix hit. 

    Configuration:
        * max_num_seqs: 2
        * number of prompts: 3
            * 0: len = 192,  max tokens = 2, step joining = 0
            * 1: len = 192, max tokens = 2, step joining = 3
            * 2: len = 192, max tokens = 2, step joining = 6
    """
    monkeypatch.setenv("VLLM_SPYRE_CP_INTERLEAVE_STEPS", "0")

    seqs_max_tokens = [2, 2, 2]
    prompts_lengths = [192, 192, 192]
    # sequence 1 joins only at step 3, when seq 0 has already finished
    # sequence 2 joins only at step 6, when seq 1 has already finished
    steps_add_reqs = [0, 3, 6]
    seeds = [0, 1, 0]  # 1st and 3rd sequence are the same

    checked_steps = [
        {
            "step": 0,
            "tkv": 0,
            "waiting": ["0"],
            "running": [],
            "request_outputs": [],
            "n_reserved_blocks": 0,
            "n_used_blocks": 0
        },
        {   # prefill chunk 1 seq 0
            "step": 1,
            "tkv": 192,
            "waiting": [],
            "running": ["0"],
            "request_outputs": [],
            "n_reserved_blocks": 4,
            "n_used_blocks": 3,
            "n_prefix_hits": 0,
        },
        {   # prefill chunk 2 seq 0
            "step": 2,
            "tkv": 192,
            "waiting": [],
            "running": ["0"],
            "request_outputs": ["0"],
            "n_reserved_blocks": 4,
            "n_used_blocks": 3,
            "n_prefix_hits": 0,
        },
        {
            # Decode 1 of request 0
            # request 1 joined the waiting queue
            "step": 3,
            "tkv": 193,
            "waiting": ["1"],
            "running": [],
            "request_outputs": ["0"],
            "finished_requests": ["0"],
            "n_reserved_blocks": 4,
            "n_used_blocks": 4
        },
        {   # prefill chunk 1 seq 1
            "step": 4,
            "tkv": 192,
            "waiting": [],
            "running": ["1"],
            "request_outputs": [],
            "n_reserved_blocks": 4,
            "n_used_blocks": 3,
            "n_prefix_hits": 0,
        },
        {   # prefill chunk 2 seq 1
            "step": 5,
            "tkv": 192,
            "waiting": [],
            "running": ["1"],
            "request_outputs": ["1"],
            "n_reserved_blocks": 4,
            "n_used_blocks": 3,
            "n_prefix_hits": 0,
        },
        {
            # Decode 1 of request 1
            # request 2 joined the waiting queue
            "step": 6,
            "tkv": 193,
            "waiting": ['2'],
            "running": [],
            "request_outputs": ["1"],
            "finished_requests": ["1"],
            "n_reserved_blocks": 4,
            "n_used_blocks": 4
        },
        {   # prefill chunk 1 seq 2
            # no prefix hit as KV cache is already overwritten!
            "step": 7,
            "tkv": 192,
            "waiting": [],
            "running": ["2"],
            "request_outputs": [],
            "n_reserved_blocks": 4,
            "n_used_blocks": 3,
            "n_prefix_hits": 0,
        },
        {   # prefill chunk 2 seq 2
            "step": 8,
            "tkv": 192,
            "waiting": [],
            "running": ["2"],
            "request_outputs": ["2"],
            "n_reserved_blocks": 4,
            "n_used_blocks": 3,
            "n_prefix_hits": 0,
        },
        {
            # Decode 1 of request 2
            "step": 9,
            "tkv": 193,
            "waiting": [],
            "running": [],
            "request_outputs": ["2"],
            "finished_requests": ["2"],
            "n_reserved_blocks": 4,
            "n_used_blocks": 4
        },
        {
            # Tkv should be cleared one step later
            "step": 10,
            "tkv": 0,
            "waiting": [],
            "running": [],
            "request_outputs": [],
            "n_reserved_blocks": 0,
            "n_used_blocks": 0
        },
    ]

    check_scheduler_inference_steps(
        model=model,
        backend=backend,
        monkeypatch=monkeypatch,
        seqs_max_tokens=seqs_max_tokens,
        prompts_lengths=prompts_lengths,
        steps_add_reqs=steps_add_reqs,
        checked_steps=checked_steps,
        max_num_seqs=max_num_seqs,
        max_model_len=max_model_len,
        available_blocks=available_blocks,
        use_cb=False,
        random_prompts=True,
        max_num_batched_tokens=max_num_batched_tokens,
        prefix_caching=True,
        seeds=seeds)


@pytest.mark.cpu
@pytest.mark.chunked_prefill
@pytest.mark.full_model
@pytest.mark.prefix_caching
# These values are all parameterized for test sorting
@pytest.mark.parametrize("max_num_seqs", [2])
@pytest.mark.parametrize("max_model_len", [256])
@pytest.mark.parametrize("max_num_batched_tokens", [128])
@pytest.mark.parametrize("available_blocks", [8])
def test_limit_blocks_prefix_hit(model: ModelInfo, backend: str,
                                 monkeypatch: pytest.MonkeyPatch,
                                 set_random_seed, max_num_seqs: int,
                                 max_model_len: int,
                                 max_num_batched_tokens: int,
                                 available_blocks: int):
    """ Scenario where three sequences are scheduled with the 1st and 3rd
    sequences being identical. While prefilling the third sequence we 
    have a prefix cache hit for the first chunk as the KV cache is still
    persistent. This is because the number of available blocks (8) is high
    enough. Note: When decreasing the number of available blocks to 4, see
    test_limit_blocks_no_prefix_hit, the same test results in a no prefix hit. 

    Configuration:
        * max_num_seqs: 2
        * number of prompts: 3
            * 0: len = 192,  max tokens = 2, step joining = 0
            * 1: len = 192, max tokens = 2, step joining = 3
            * 2: len = 192, max tokens = 2, step joining = 6
    """
    monkeypatch.setenv("VLLM_SPYRE_CP_INTERLEAVE_STEPS", "0")

    seqs_max_tokens = [2, 2, 2]
    prompts_lengths = [192, 192, 192]
    # sequence 1 joins only at step 3, when seq 0 has already finished
    # sequence 2 joins only at step 6, when seq 1 has already finished
    steps_add_reqs = [0, 3, 6]
    seeds = [0, 1, 0]  # 1st and 3rd sequence are the same

    checked_steps = [
        {
            "step": 0,
            "tkv": 0,
            "waiting": ["0"],
            "running": [],
            "request_outputs": [],
            "n_reserved_blocks": 0,
            "n_used_blocks": 0
        },
        {   # prefill chunk 1 seq 0
            "step": 1,
            "tkv": 192,
            "waiting": [],
            "running": ["0"],
            "request_outputs": [],
            "n_reserved_blocks": 4,
            "n_used_blocks": 3,
            "n_prefix_hits": 0,
        },
        {   # prefill chunk 2 seq 0
            "step": 2,
            "tkv": 192,
            "waiting": [],
            "running": ["0"],
            "request_outputs": ["0"],
            "n_reserved_blocks": 4,
            "n_used_blocks": 3,
            "n_prefix_hits": 0,
        },
        {
            # Decode 1 of request 0
            # request 1 joined the waiting queue
            "step": 3,
            "tkv": 193,
            "waiting": ["1"],
            "running": [],
            "request_outputs": ["0"],
            "finished_requests": ["0"],
            "n_reserved_blocks": 4,
            "n_used_blocks": 4
        },
        {   # prefill chunk 1 seq 1
            "step": 4,
            "tkv": 192,
            "waiting": [],
            "running": ["1"],
            "request_outputs": [],
            "n_reserved_blocks": 4,
            "n_used_blocks": 3,
            "n_prefix_hits": 0,
        },
        {   # prefill chunk 2 seq 1
            "step": 5,
            "tkv": 192,
            "waiting": [],
            "running": ["1"],
            "request_outputs": ["1"],
            "n_reserved_blocks": 4,
            "n_used_blocks": 3,
            "n_prefix_hits": 0,
        },
        {
            # Decode 1 of request 1
            # request 2 joined the waiting queue
            "step": 6,
            "tkv": 193,
            "waiting": ['2'],
            "running": [],
            "request_outputs": ["1"],
            "finished_requests": ["1"],
            "n_reserved_blocks": 4,
            "n_used_blocks": 4
        },
        {   # prefill chunk 1 seq 2
            # prefix hit as KV cache is still persistent
            "step": 7,
            "tkv": 192,
            "waiting": [],
            "running": ["2"],
            "request_outputs": [],
            "n_reserved_blocks": 4,
            "n_used_blocks": 3,
            "n_prefix_hits": 1,
            "n_cached_blocks": 1
        },
        {   # prefill chunk 2 seq 2
            "step": 8,
            "tkv": 192,
            "waiting": [],
            "running": ["2"],
            "request_outputs": ["2"],
            "n_reserved_blocks": 4,
            "n_used_blocks": 3,
            "n_prefix_hits": 0,
            "n_cached_blocks": 1
        },
        {
            # Decode 1 of request 2
            "step": 9,
            "tkv": 193,
            "waiting": [],
            "running": [],
            "request_outputs": ["2"],
            "finished_requests": ["2"],
            "n_reserved_blocks": 4,
            "n_used_blocks": 4,
            "n_cached_blocks": 1
        },
        {
            # Tkv should be cleared one step later
            "step": 10,
            "tkv": 0,
            "waiting": [],
            "running": [],
            "request_outputs": [],
            "n_reserved_blocks": 0,
            "n_used_blocks": 0
        },
    ]

    check_scheduler_inference_steps(
        model=model,
        backend=backend,
        monkeypatch=monkeypatch,
        seqs_max_tokens=seqs_max_tokens,
        prompts_lengths=prompts_lengths,
        steps_add_reqs=steps_add_reqs,
        checked_steps=checked_steps,
        max_num_seqs=max_num_seqs,
        max_model_len=max_model_len,
        available_blocks=available_blocks,
        use_cb=False,
        random_prompts=True,
        max_num_batched_tokens=max_num_batched_tokens,
        prefix_caching=True,
        seeds=seeds)


@pytest.mark.cpu
@pytest.mark.chunked_prefill
@pytest.mark.full_model
@pytest.mark.prefix_caching
# These values are all parameterized for test sorting
@pytest.mark.parametrize("max_num_seqs", [2])
@pytest.mark.parametrize("max_model_len", [512])
@pytest.mark.parametrize("max_num_batched_tokens", [128])
@pytest.mark.parametrize("available_blocks", [None])
def test_full_match(model: ModelInfo, backend: str,
                    monkeypatch: pytest.MonkeyPatch, set_random_seed,
                    max_num_seqs: int, max_model_len: int,
                    max_num_batched_tokens: int, available_blocks: int):
    """ Scenario where two equal sequences are scheduled.
    Both sequences have exactly 3 chunks worth of tokens, thus
    resulting in a 100% match up to the last token. This test
    makes sure that the last chunk is not reused.

    Configuration:
        * max_num_seqs: 2
        * number of prompts: 2
            * 0: len = 384,  max tokens = 2, step joining = 0
            * 1: len = 384, max tokens = 2, step joining = 0
    """
    monkeypatch.setenv("VLLM_SPYRE_CP_INTERLEAVE_STEPS", "0")

    seqs_max_tokens = [2, 2]
    prompts_lengths = [384, 384]
    steps_add_reqs = [0, 0]
    seeds = [0, 0]  # twice the same sequence

    checked_steps = [
        {
            "step": 0,
            "tkv": 0,
            "waiting": ["0", "1"],
            "running": [],
            "request_outputs": [],
            "n_reserved_blocks": 0,
            "n_used_blocks": 0
        },
        {   # prefill chunk 1 seq 0
            "step": 1,
            "tkv": 384,
            "waiting": ["1"],
            "running": ["0"],
            "request_outputs": [],
            "n_reserved_blocks": 7,
            "n_used_blocks": 6,
            "n_prefix_hits": 0,
        },
        {   # prefill chunk 2 seq 0
            "step": 2,
            "tkv": 384,
            "waiting": ["1"],
            "running": ["0"],
            "request_outputs": [],
            "n_reserved_blocks": 7,
            "n_used_blocks": 6,
            "n_prefix_hits": 0,
        },
        {   # prefill chunk 3 seq 0
            "step": 3,
            "tkv": 384,
            "waiting": ["1"],
            "running": ["0"],
            "request_outputs": ["0"],
            "n_reserved_blocks": 7,
            "n_used_blocks": 6,
            "n_prefix_hits": 0,
        },
        {   # prefill chunk 1 seq 1
            # prefix hit!
            "step": 4,
            "tkv": 384,
            "waiting": [],
            "running": ["1", "0"],
            "request_outputs": [],
            "n_reserved_blocks": 14,
            "n_used_blocks": 12,
            "n_prefix_hits": 1,
            # The number of cached blocks is determined up front
            "n_cached_blocks": 4
        },
        {   # prefill chunk 2 seq 1
            # cannot use prefix, as the last chunk has to always be recomputed
            "step": 5,
            "tkv": 384,
            "waiting": [],
            "running": ["1", "0"],
            "request_outputs": [],
            "n_reserved_blocks": 14,
            "n_used_blocks": 12,
            "n_prefix_hits": 1,
            "n_cached_blocks": 4
        },
        {   # prefill chunk 3 seq 1
            # cannot use prefix, as the last chunk has to always be recomputed
            "step": 6,
            "tkv": 384,
            "waiting": [],
            "running": ["1", "0"],
            "request_outputs": ["1"],
            "n_reserved_blocks": 14,
            "n_used_blocks": 12,
            "n_prefix_hits": 0,
            "n_cached_blocks": 4
        },
        {
            # Decode 1 of request 0.
            # Decode 1 of request 1.
            "step": 7,
            "tkv": 385,
            "waiting": [],
            "running": [],
            "request_outputs": ["1", "0"],
            "finished_requests": ["1", "0"],
            "n_reserved_blocks": 14,
            "n_used_blocks": 14,
            "n_cached_blocks": 4
        },
        {
            # Tkv should be cleared one step later
            "step": 8,
            "tkv": 0,
            "waiting": [],
            "running": [],
            "request_outputs": [],
            "n_reserved_blocks": 0,
            "n_used_blocks": 0
        },
    ]

    check_scheduler_inference_steps(
        model=model,
        backend=backend,
        monkeypatch=monkeypatch,
        seqs_max_tokens=seqs_max_tokens,
        prompts_lengths=prompts_lengths,
        steps_add_reqs=steps_add_reqs,
        checked_steps=checked_steps,
        max_num_seqs=max_num_seqs,
        max_model_len=max_model_len,
        available_blocks=available_blocks,
        use_cb=False,
        random_prompts=True,
        max_num_batched_tokens=max_num_batched_tokens,
        prefix_caching=True,
        seeds=seeds,
    )



"""Compare graphs generated by vLLM-Spyre vs AFTU

Run `python -m pytest tests/aftu/test_compare_graphs.py`.
"""

import os
import sys
import tempfile

import pytest
import torch
from graph_compare_utils import (collect_graph_files, compare_graphs,
                                 get_model_path,
                                 run_inference_py_and_get_graphs)
from pytest_mock.plugin import MockerFixture
from spyre_util import DecodeWarmupShapes, ModelInfo, patch_environment
from vllm import LLM

from vllm_spyre.model_executor.model_loader.spyre import SpyreCausalLM


# TODO: AFTU uses float32 for mask, while we use float16, which
# can generates different graphs. For now, until AFTU does not change
# this, we can just mock this method to match with AFTU behavior.
# NOTE: we need to set VLLM_ENABLE_V1_MULTIPROCESSING=0 otherwise this
# mock will not propagate to the child process of the model runner
def mock_get_mask_dtype(mocker: MockerFixture):

    mocker.patch.object(SpyreCausalLM,
                        "get_mask_dtype",
                        return_value=torch.float32)


@pytest.mark.spyre
@pytest.mark.cb
def test_compare_graphs_cb(model: ModelInfo, max_num_seqs: int,
                           max_model_len: int, monkeypatch: pytest.MonkeyPatch,
                           mocker: MockerFixture):
    """Test if the graphs generated by vllm-spyre matches with the ones by
    AFTU in continuous batching mode """

    model_path = get_model_path(model)

    attn_type = 'paged_fp8' if model.is_quantized \
        else 'paged'

    if model.is_quantized:
        mock_get_mask_dtype(mocker)

    inference_py_args = [
        sys.executable, "-m", "aiu_fms_testing_utils.scripts.inference",
        "--architecture", "hf_pretrained", "--model_path", model_path,
        "--tokenizer", model_path, "--unfuse_weights", "--device_type", "aiu",
        "--compile", "--cast_bf16_to_fp16", "--compile_dynamic",
        "--min_pad_length", "64", "--max_new_tokens", "5", "--batch_size",
        str(max_num_seqs), "--compile_dynamic_sendnn", "--attention_type",
        attn_type
    ]

    if not model.is_quantized:
        inference_py_args += ["--default_dtype", "fp16"]

    extra_env = {
        "VLLM_DT_MAX_CONTEXT_LEN": str(max_model_len),
        "VLLM_DT_MAX_BATCH_SIZE": str(max_num_seqs),
        "VLLM_DT_MAX_BATCH_TKV_LIMIT": str(1024 * 128)
    }
    aftu_graphs = run_inference_py_and_get_graphs(inference_py_args, extra_env)

    assert len(aftu_graphs) > 0

    # VLLM

    monkeypatch.setenv("DEE_DUMP_GRAPHS", "vllm_cb")
    # Disable cache to produce the graphs
    monkeypatch.setenv("TORCH_SENDNN_CACHE_ENABLE", "0")

    # need for the mocker
    monkeypatch.setenv('VLLM_ENABLE_V1_MULTIPROCESSING', 0)
    patch_environment(use_cb=True,
                      warmup_shapes=None,
                      backend="sendnn",
                      monkeypatch=monkeypatch)

    original_cwd = os.getcwd()
    try:
        # Change to temp dir to set the test environment clean
        with tempfile.TemporaryDirectory() as tmpdir:
            os.chdir(tmpdir)

            # We only need to load the model
            LLM(model=model.name,
                revision=model.revision,
                max_model_len=max_model_len,
                tensor_parallel_size=1,
                max_num_seqs=max_num_seqs)

            vllm_graphs = collect_graph_files(tmpdir)
    finally:
        # Restore in case of exception
        os.chdir(original_cwd)

    assert compare_graphs(vllm_graphs, aftu_graphs)


@pytest.mark.spyre
@pytest.mark.parametrize(
    "warmup_shapes", [[(64, 4, 4)]])  # (prompt_length/new_tokens/batch_size)
def test_compare_graphs_static_batching(model: ModelInfo,
                                        warmup_shapes: DecodeWarmupShapes,
                                        monkeypatch: pytest.MonkeyPatch,
                                        mocker: MockerFixture) -> None:
    """Test if the graphs generated by vllm-spyre matches with the ones by
    AFTU in static batching mode """

    attn_type = 'math_fp8' if model.is_quantized \
        else 'sdpa'

    if model.is_quantized:
        mock_get_mask_dtype(mocker)

    model_path = get_model_path(model)

    inference_py_args = [
        sys.executable, "-m", "aiu_fms_testing_utils.scripts.inference",
        "--architecture", "hf_pretrained", "--model_path", model_path,
        "--tokenizer", model_path, "--unfuse_weights", "--device_type", "aiu",
        "--compile", "--cast_bf16_to_fp16", "--compile_dynamic",
        "--fixed_prompt_length",
        str(warmup_shapes[0][0]), "--max_new_tokens",
        str(warmup_shapes[0][1]), "--batch_size",
        str(warmup_shapes[0][2]), "--attention_type", attn_type
    ]

    if not model.is_quantized:
        inference_py_args += ["--default_dtype", "fp16"]

    aftu_graphs = run_inference_py_and_get_graphs(inference_py_args)
    assert len(aftu_graphs) > 0
    # VLLM

    monkeypatch.setenv("DEE_DUMP_GRAPHS", "vllm_sb")
    # Disable cache to produce the graphs
    monkeypatch.setenv("TORCH_SENDNN_CACHE_ENABLE", "0")
    # needed for the mocker
    monkeypatch.setenv('VLLM_ENABLE_V1_MULTIPROCESSING', 0)
    patch_environment(use_cb=False,
                      warmup_shapes=warmup_shapes,
                      backend="sendnn",
                      monkeypatch=monkeypatch)

    original_cwd = os.getcwd()
    try:
        # Change to temp dir to set the test environment clean
        with tempfile.TemporaryDirectory() as tmpdir:
            os.chdir(tmpdir)

            LLM(model=model.name,
                revision=model.revision,
                max_model_len=2048,
                tensor_parallel_size=1,
                max_num_seqs=warmup_shapes[0][2])

            vllm_graphs = collect_graph_files(tmpdir)
    finally:
        # Restore in case of exception
        os.chdir(original_cwd)

    assert compare_graphs(vllm_graphs, aftu_graphs)


@pytest.mark.spyre
@pytest.mark.chunked_prefill
def test_compare_graphs_chunked_prefill(model: ModelInfo, max_num_seqs: int,
                                        max_model_len: int,
                                        monkeypatch: pytest.MonkeyPatch):
    """Test if the graphs generated by vllm-spyre matches with the ones by
    AFTU with chunked prefill enabled """

    if model.is_quantized:
        pytest.skip("Quantized model are not yet supported "
                    "with chunked prefill")
    model_path = get_model_path(model)

    chunk_size = 128
    inference_py_args = [
        sys.executable,
        "-m",
        "aiu_fms_testing_utils.scripts.inference",
        "--architecture",
        "hf_pretrained",
        "--model_path",
        model_path,
        "--tokenizer",
        model_path,
        "--unfuse_weights",
        "--device_type",
        "aiu",
        "--compile",
        "--cast_bf16_to_fp16",
        "--compile_dynamic",
        "--min_pad_length",
        "64",
        "--max_new_tokens",
        "5",
        "--batch_size",
        str(max_num_seqs),
        "--compile_dynamic_sendnn",
        "--attention_type",
        'paged',
        "--default_dtype",
        "fp16",
        "--prefill_chunk_size",
        str(chunk_size),
    ]

    extra_env = {
        "VLLM_DT_MAX_CONTEXT_LEN": str(max_model_len),
        "VLLM_DT_MAX_BATCH_SIZE": str(max_num_seqs),
        "VLLM_DT_MAX_BATCH_TKV_LIMIT": str(1024 * 128),
        "VLLM_DT_CHUNK_LEN": str(chunk_size)
    }
    aftu_graphs = run_inference_py_and_get_graphs(inference_py_args, extra_env)

    assert len(aftu_graphs) > 0

    # VLLM

    monkeypatch.setenv("DEE_DUMP_GRAPHS", "vllm_cb_cp")
    # Disable cache to produce the graphs
    monkeypatch.setenv("TORCH_SENDNN_CACHE_ENABLE", "0")

    monkeypatch.setenv('VLLM_DT_CHUNK_LEN', str(chunk_size))
    patch_environment(use_cb=True,
                      warmup_shapes=None,
                      backend="sendnn",
                      monkeypatch=monkeypatch,
                      use_chunked_prefill=True)

    original_cwd = os.getcwd()
    try:
        # Change to temp dir to set the test environment clean
        with tempfile.TemporaryDirectory() as tmpdir:
            os.chdir(tmpdir)

            # We only need to load the model
            LLM(model=model.name,
                revision=model.revision,
                max_model_len=max_model_len,
                tensor_parallel_size=1,
                max_num_batched_tokens=chunk_size,
                max_num_seqs=max_num_seqs)

            vllm_graphs = collect_graph_files(tmpdir)
    finally:
        # Restore in case of exception
        os.chdir(original_cwd)

    assert compare_graphs(vllm_graphs, aftu_graphs)



"""Verification of vLLM output by comparing with HF

Run `python -m pytest tests/e2e/test_spyre_basic.py`.
"""

from typing import cast
from unittest.mock import patch

import pytest
from llm_cache import get_cached_llm
from output_util import (compare_results, extract_output, generate_hf_output,
                         setup_golden_token)
from pytest_mock.plugin import MockerFixture
from spyre_util import (ModelInfo, get_chicken_soup_prompts,
                        get_longer_chicken_soup_prompts)
from vllm import LLM, SamplingParams

from vllm_spyre.v1.worker.spyre_model_runner import SamplingForwardInputs


def get_model_runner(cp_model: LLM):
    return cp_model.llm_engine.\
        engine_core.engine_core.model_executor.\
            driver_worker.worker.model_runner


chicken_soup_prompts = get_longer_chicken_soup_prompts(4)

# NOTE: considering granite 3.3 tokenizer
# Should have 51 tokens
prompt_51 = get_chicken_soup_prompts(1)[0]
# Should have 95 tokens
prompt_95 = chicken_soup_prompts[0]
# Should have 251
prompt_251 = chicken_soup_prompts[0] + chicken_soup_prompts[
    1] + chicken_soup_prompts[2]
# Should have 260 tokens
prompt_260 = chicken_soup_prompts[0] + chicken_soup_prompts[2] + \
    chicken_soup_prompts[3]

USE_CASES = {
    # Case I - Prompt fits in a single chunk
    "case_Ia": (prompt_95, 128, 1, 0),
    # Case I - Prompt fits in a single chunk with left padding
    "case_Ib": (prompt_51, 128, 1, 64),
    # Case II - Has left padding
    "case_II": (prompt_260, 128, 3, 64),
    # Case III again - no padding
    "case_III": (prompt_251, 128, 2, 0),
}


@pytest.mark.chunked_prefill
@pytest.mark.parametrize("use_case", list(USE_CASES.keys()))
def test_chunked_prefill_correctness(model: ModelInfo, backend: str,
                                     max_num_seqs: int, max_model_len: int,
                                     monkeypatch: pytest.MonkeyPatch,
                                     mocker: MockerFixture, use_case: str,
                                     use_llm_cache) -> None:
    """
    Minimal test to check if vllm-spyre activate code for chunked prefill for
    a prompt greater than the chunk size
    """


    (prompt, chunk_size, expected_chunk_count, expected_left_padding) =\
          USE_CASES[use_case]
    max_new_tokens = 8

    hf_outputs = generate_hf_output(
        model=model,
        prompts=[prompt],
        max_new_tokens=max_new_tokens,
        ignore_eos=True,
    )
    ### NB: May not be guaranteed to be set
    monkeypatch.setenv("VLLM_ENABLE_V1_MULTIPROCESSING", "0")

    cp_model = get_cached_llm(model=model,
                              max_model_len=max_model_len,
                              tensor_parallel_size=1,
                              backend=backend,
                              monkeypatch=monkeypatch,
                              max_num_seqs=max_num_seqs,
                              use_cb=True,
                              max_num_batched_tokens=chunk_size)
    model_runner = get_model_runner(cp_model)

    sampling_params = SamplingParams(max_tokens=max_new_tokens,
                                     temperature=0,
                                     logprobs=0,
                                     ignore_eos=True)
    gti_sampling_params = setup_golden_token(model, sampling_params,
                                             hf_outputs)

    _prepare_chunked_prefill = model_runner._prepare_chunked_prefill
    records = []

    def wrapper(self, *args, **kwargs):
        model_input = \
            _prepare_chunked_prefill(self, *args, **kwargs)
        records.append(model_input)
        return model_input

    with patch.object(model_runner, "_prepare_chunked_prefill",
                      wraps=wrapper) as spy:
        results = cp_model.generate(prompt, gti_sampling_params)
        vllm_results = [extract_output(results[0])]

        for r in records:
            model_input = cast(SamplingForwardInputs, r)
            # Must be a single value
            left_padding = model_input.left_padded_prompt_mask[0].item()
            assert left_padding == expected_left_padding

    # Validate if the prefill was chunked
    spy.assert_called()
    assert spy.call_count == expected_chunk_count

    # Validate output
    compare_results(model=model,
                    tensor_parallel_size=1,
                    backend=backend,
                    vllm_results=vllm_results,
                    hf_results=hf_outputs,
                    prompts=[prompt])



import os

import pytest
from vllm.v1.core.sched.output import NewRequestData, SchedulerOutput
from vllm.v1.kv_cache_interface import FullAttentionSpec

from vllm_spyre.compat_utils import dataclass_fields

pytestmark = pytest.mark.compat

VLLM_VERSION = os.getenv("TEST_VLLM_VERSION", "default")


def test_mm_inputs():

    if VLLM_VERSION == "vLLM:lowest":
        # Can remove "mm_kwargs", "mm_hashes", "mm_positions"
        # (replaced by mm_features)
        assert 'mm_kwargs' in dataclass_fields(NewRequestData)


def test_get_sampler():
    if VLLM_VERSION == "vLLM:lowest":
        try:
            from vllm.model_executor.layers.sampler import (  # # noqa
                get_sampler)
        except ImportError as e:
            raise AssertionError(
                "Remove backwards compatibility for get_sampler") from e


def test_use_mla():
    if VLLM_VERSION == "vLLM:lowest":
        # Can remove backwards compatibility for use_mla
        assert "use_mla" in dataclass_fields(FullAttentionSpec)


def test_pin_memory_available():
    if VLLM_VERSION == "vLLM:lowest":
        try:
            from vllm.utils import is_pin_memory_available  # # noqa
            from vllm.utils import make_tensor_with_pad  # # noqa
        except ImportError as e:
            raise AssertionError("remove backwards compatibility imports for "
                                 "is_pin_memory_available and "
                                 "make_tensor_with_pad") from e


def test_multi_modal_cache_stats():
    if VLLM_VERSION == "vLLM:lowest":
        # If this import succeeds then remove the backwards compatibility type
        # def for MultiModalCacheStats
        with pytest.raises(ImportError):
            from vllm.v1.metrics.stats import MultiModalCacheStats  # # noqa


def test_v0_worker_base():
    if VLLM_VERSION == "vLLM:lowest":
        try:
            from vllm.worker.worker_base import WorkerBase  # # noqa
        except ImportError as e:
            raise AssertionError(
                "remove the backwards compatibility code from "
                "the SpyreWorker initializer") from e


def test_structured_output_request_ids():
    if VLLM_VERSION == "vLLM:lowest":
        # Can remove "structured_output_request_ids" and "grammar_bitmask"
        # from backwards compat
        assert 'structured_output_request_ids' in dataclass_fields(
            SchedulerOutput)



import logging
import os

import pytest
from spyre_util import (DecodeWarmupShapes, patch_warmup_shapes,
                        write_sample_model_config)
from vllm.config import (ModelConfig, ParallelConfig, SchedulerConfig,
                         VllmConfig)

from vllm_spyre.compilation_utils import PRE_COMPILE_MODEL_CATALOG_FILENAME


@pytest.mark.precompilation
@pytest.mark.parametrize("batch_type", ["sb", "cb"])
def test_handle_disable_compilation(model, caplog_vllm_spyre, monkeypatch,
                                    tmp_path, batch_type):
    """
    Test handle_disable_compilation for static and continuous batching.
    Note: since the validation here is only giving warning in case of mismatch,
    these tests will only fail if there is a bug in the logic
    """

    # Patch version to avoid test failures around version mismatch
    monkeypatch.setattr("vllm_spyre._version.version", "0.8.0")

    if batch_type == "sb":
        patch_warmup_shapes(DecodeWarmupShapes([(128, 128, 1)]), monkeypatch)
        sample_model_config = {
            "vllm_spyre_version": "0.8.0",
            "data": {
                "MODEL_NAME": "/models/granite-3.3-8b-instruct-FP8",
                "NUM_AIUS": 2,
                "VLLM_SPYRE_WARMUP_PROMPT_LENS": "128",
                "VLLM_SPYRE_WARMUP_NEW_TOKENS": "128",
                "VLLM_SPYRE_WARMUP_BATCH_SIZES": "1",
            },
        }

    else:
        monkeypatch.setenv("VLLM_SPYRE_USE_CB", "1")
        sample_model_config = {
            "vllm_spyre_version": "0.8.0",
            "data": {
                "MODEL_NAME": "/models/granite-3.3-8b-instruct-FP8",
                "NUM_AIUS": 2,
                "VLLM_DT_MAX_CONTEXT_LEN": 256,
                "VLLM_DT_MAX_BATCH_SIZE": 2
            },
        }

    write_sample_model_config(tmp_path, sample_model_config)

    monkeypatch.setenv("VLLM_SPYRE_REQUIRE_PRECOMPILED_DECODERS", "1")
    monkeypatch.setenv("TORCH_SENDNN_CACHE_DIR", str(tmp_path))
    monkeypatch.setenv("TORCH_SENDNN_CACHE_ENABLE", "1")
    # Register the DISABLE_COMPILATION env variable with monkeypatch so that
    # it resets the value to its previous state after the
    # test as a cleanup
    monkeypatch.setenv("DISABLE_COMPILATION", "")

    with caplog_vllm_spyre.at_level(logging.INFO):
        _ = VllmConfig(model_config=ModelConfig(model=model.name,
                                                revision=model.revision,
                                                max_model_len=256),
                       parallel_config=ParallelConfig(tensor_parallel_size=2),
                       scheduler_config=SchedulerConfig(max_num_seqs=2))
        assert "[PRECOMPILED_WARN] Setting DISABLE_COMPILATION" \
            in caplog_vllm_spyre.text

    assert "DISABLE_COMPILATION" in os.environ
    assert os.getenv("DISABLE_COMPILATION") == "true"


@pytest.mark.precompilation
@pytest.mark.parametrize("batch_type", ["sb", "cb"])
def test_handle_disable_compilation_catalog(model, caplog_vllm_spyre,
                                            monkeypatch, tmp_path, batch_type):
    """
    Test handle_disable_compilation for static and continuous batching.
    Note: since the validation here is only giving warning in case of mismatch,
    these tests will only fail if there is a bug in the logic
    """

    # Patch version to avoid test failures around version mismatch
    monkeypatch.setattr("vllm_spyre._version.version", "0.8.0")

    if batch_type == "sb":
        monkeypatch.setenv("VLLM_SPYRE_WARMUP_PROMPT_LENS", "128")
        monkeypatch.setenv("VLLM_SPYRE_WARMUP_NEW_TOKENS", "128")
        monkeypatch.setenv("VLLM_SPYRE_WARMUP_BATCH_SIZES", "1")
        monkeypatch.setenv("VLLM_SPYRE_USE_CB", "0")
        sample_model_config1 = {
            "vllm_spyre_version": "0.8.0",
            "data": {
                "MODEL_NAME": "/models/granite-3.3-8b-instruct-FP8",
                "NUM_AIUS": 2,
                "VLLM_SPYRE_WARMUP_PROMPT_LENS": "128",
                "VLLM_SPYRE_WARMUP_NEW_TOKENS": "128",
                "VLLM_SPYRE_WARMUP_BATCH_SIZES": "1",
            },
        }
        sample_model_config2 = {
            "vllm_spyre_version": "0.8.0",
            "data": {
                "MODEL_NAME": "/models/granite-3.3-8b-instruct-FP8",
                "NUM_AIUS": 2,
                "VLLM_SPYRE_WARMUP_PROMPT_LENS": "256",
                "VLLM_SPYRE_WARMUP_NEW_TOKENS": "256",
                "VLLM_SPYRE_WARMUP_BATCH_SIZES": "1",
            },
        }

    else:
        monkeypatch.setenv("VLLM_SPYRE_USE_CB", "1")
        sample_model_config1 = {
            "vllm_spyre_version": "0.8.0",
            "data": {
                "MODEL_NAME": "/models/granite-3.3-8b-instruct-FP8",
                "NUM_AIUS": 2,
                "VLLM_DT_MAX_CONTEXT_LEN": 256,
                "VLLM_DT_MAX_BATCH_SIZE": 2
            },
        }
        sample_model_config2 = {
            "vllm_spyre_version": "0.8.0",
            "data": {
                "MODEL_NAME": "/models/granite-3.3-8b-instruct-FP8",
                "NUM_AIUS": 2,
                "VLLM_DT_MAX_CONTEXT_LEN": 512,
                "VLLM_DT_MAX_BATCH_SIZE": 2
            },
        }

    sample_model_config = [sample_model_config1, sample_model_config2]

    write_sample_model_config(tmp_path,
                              sample_model_config,
                              filename=PRE_COMPILE_MODEL_CATALOG_FILENAME)

    monkeypatch.setenv("VLLM_SPYRE_REQUIRE_PRECOMPILED_DECODERS", "1")
    monkeypatch.setenv("TORCH_SENDNN_CACHE_DIR", str(tmp_path))
    monkeypatch.setenv("TORCH_SENDNN_CACHE_ENABLE", "1")
    # Register the DISABLE_COMPILATION env variable with monkeypatch so that
    # it resets the value to its previous state after the
    # test as a cleanup
    monkeypatch.setenv("DISABLE_COMPILATION", "")

    with caplog_vllm_spyre.at_level(logging.INFO):
        _ = VllmConfig(model_config=ModelConfig(model=model.name,
                                                revision=model.revision,
                                                max_model_len=256),
                       parallel_config=ParallelConfig(tensor_parallel_size=2),
                       scheduler_config=SchedulerConfig(max_num_seqs=2))

        assert "[PRECOMPILED_WARN] Setting DISABLE_COMPILATION" \
            in caplog_vllm_spyre.text

    assert "DISABLE_COMPILATION" in os.environ
    assert os.getenv("DISABLE_COMPILATION") == "true"


@pytest.mark.precompilation
@pytest.mark.parametrize("batch_type", ["sb", "cb"])
def test_catalog_config_mismatch(model, caplog_vllm_spyre, monkeypatch,
                                 tmp_path, batch_type):
    """
    Test handle_disable_compilation for static and continuous batching
    and verify if we get proper error in case of mismatch catalog file
    """

    # Patch version to avoid test failures around version mismatch
    monkeypatch.setattr("vllm_spyre._version.version", "0.8.0")

    if batch_type == "sb":
        monkeypatch.setenv("VLLM_SPYRE_WARMUP_PROMPT_LENS", "64")
        monkeypatch.setenv("VLLM_SPYRE_WARMUP_NEW_TOKENS", "128")
        monkeypatch.setenv("VLLM_SPYRE_WARMUP_BATCH_SIZES", "1")
        monkeypatch.setenv("VLLM_SPYRE_USE_CB", "0")
        sample_model_config1 = {
            "vllm_spyre_version": "0.8.0",
            "data": {
                "MODEL_NAME": "/models/granite-3.3-8b-instruct-FP8",
                "NUM_AIUS": 2,
                "VLLM_SPYRE_WARMUP_PROMPT_LENS": "128",
                "VLLM_SPYRE_WARMUP_NEW_TOKENS": "128",
                "VLLM_SPYRE_WARMUP_BATCH_SIZES": "1",
            },
        }
        sample_model_config2 = {
            "vllm_spyre_version": "0.8.0",
            "data": {
                "MODEL_NAME": "/models/granite-3.3-8b-instruct-FP8",
                "NUM_AIUS": 2,
                "VLLM_SPYRE_WARMUP_PROMPT_LENS": "256",
                "VLLM_SPYRE_WARMUP_NEW_TOKENS": "256",
                "VLLM_SPYRE_WARMUP_BATCH_SIZES": "1",
            },
        }

    else:
        monkeypatch.setenv("VLLM_SPYRE_USE_CB", "1")
        sample_model_config1 = {
            "vllm_spyre_version": "0.8.0",
            "data": {
                "MODEL_NAME": "/models/granite-3.3-8b-instruct-FP8",
                "NUM_AIUS": 2,
                "VLLM_DT_MAX_CONTEXT_LEN": 256,
                "VLLM_DT_MAX_BATCH_SIZE": 2
            },
        }
        sample_model_config2 = {
            "vllm_spyre_version": "0.8.0",
            "data": {
                "MODEL_NAME": "/models/granite-3.3-8b-instruct-FP8",
                "NUM_AIUS": 2,
                "VLLM_DT_MAX_CONTEXT_LEN": 512,
                "VLLM_DT_MAX_BATCH_SIZE": 2
            },
        }

    sample_model_config = [sample_model_config1, sample_model_config2]

    write_sample_model_config(tmp_path,
                              sample_model_config,
                              filename=PRE_COMPILE_MODEL_CATALOG_FILENAME)

    monkeypatch.setenv("VLLM_SPYRE_REQUIRE_PRECOMPILED_DECODERS", "1")
    monkeypatch.setenv("TORCH_SENDNN_CACHE_DIR", str(tmp_path))
    monkeypatch.setenv("TORCH_SENDNN_CACHE_ENABLE", "1")
    # Register the DISABLE_COMPILATION env variable with monkeypatch so that
    # it resets the value to its previous state after the
    # test as a cleanup
    monkeypatch.setenv("DISABLE_COMPILATION", "")

    with caplog_vllm_spyre.at_level(logging.WARNING):
        _ = VllmConfig(model_config=ModelConfig(model=model.name,
                                                revision=model.revision,
                                                max_model_len=64),
                       parallel_config=ParallelConfig(tensor_parallel_size=2),
                       scheduler_config=SchedulerConfig(max_num_seqs=2))
        assert "[PRECOMPILED_WARN]" in caplog_vllm_spyre.text
        assert "doesn't match any of the pre-compiled model " \
        "configurations. Catalog:" in caplog_vllm_spyre.text

    assert "DISABLE_COMPILATION" in os.environ
    assert os.getenv("DISABLE_COMPILATION") == "true"
